import 'package:anilibria/src/enums/enums.dart';

class Title {
  final int? id;
  final String? code;
  final TitleNames? names;
  final String? announce;
  final TitleStatus? status;
  final Posters? posters;
  final DateTime? updated;
  final DateTime? lastChange;
  final TitleType? type;
  final List<String>? genres;
  final Team? team;
  final Season? season;
  final List<Franchise>? franchises;
  final String? description;
  final int? inFavorites;
  final Blocked? blocked;
  final Player? player;
  final Torrents? torrents;

  Title({
    this.id,
    this.code,
    this.names,
    this.announce,
    this.status,
    this.posters,
    this.updated,
    this.lastChange,
    this.type,
    this.genres,
    this.team,
    this.season,
    this.franchises,
    this.description,
    this.inFavorites,
    this.blocked,
    this.player,
    this.torrents,
  });

  Title.fromJson(Map<String, dynamic> json)
      : id = json['id'],
        code = json['code'],
        names =
            json['names'] == null ? null : TitleNames.fromJson(json['names']),
        announce = json['announce'],
        status = json['status'] == null
            ? null
            : TitleStatus.fromJson(json['status']),
        posters =
            json['posters'] == null ? null : Posters.fromJson(json['posters']),
        updated = json['updated'] == null
            ? null
            : DateTime.fromMillisecondsSinceEpoch(json['updated'] * 1000),
        lastChange = json['last_change'] == null
            ? null
            : DateTime.fromMillisecondsSinceEpoch(json['last_change'] * 1000),
        type = json['type'] == null ? null : TitleType.fromJson(json['type']),
        genres =
            json['genres'] == null ? null : List<String>.from(json['genres']),
        team = json['team'] == null ? null : Team.fromJson(json['team']),
        season =
            json['season'] == null ? null : Season.fromJson(json['season']),
        franchises = json['franchises'] == null
            ? null
            : [for (final e in json['franchises']) Franchise.fromJson(e)],
        description = json['description'],
        inFavorites = json['in_favorites'],
        blocked =
            json['blocked'] == null ? null : Blocked.fromJson(json['blocked']),
        player =
            json['player'] == null ? null : Player.fromJson(json['player']),
        torrents = json['torrents'] == null
            ? null
            : Torrents.fromJson(json['torrents']);
}

class TitleNames {
  final String? ru;
  final String? en;
  final String? alternative;

  TitleNames({
    this.ru,
    this.en,
    this.alternative,
  });

  TitleNames.fromJson(Map<String, dynamic> json)
      : ru = json['ru'],
        en = json['en'],
        alternative = json['alternative'];
}

class TitleStatus {
  final String? string;
  final TitleStatusCode? code;

  TitleStatus({
    this.string,
    this.code,
  });

  TitleStatus.fromJson(Map<String, dynamic> json)
      : string = json['string'],
        code = json['code'] == null
            ? null
            : TitleStatusCode.fromValue(json['code']);
}

class Posters {
  final Poster? small;
  final Poster? medium;
  final Poster? original;

  Posters({this.small, this.medium, this.original});

  Posters.fromJson(Map<String, dynamic> json)
      : small = json['small'] == null ? null : Poster.fromJson(json['small']),
        medium =
            json['medium'] == null ? null : Poster.fromJson(json['medium']),
        original =
            json['original'] == null ? null : Poster.fromJson(json['original']);
}

class Poster {
  final String? url;
  final String? rawBase64File;

  Poster({
    this.url,
    this.rawBase64File,
  });

  Poster.fromJson(Map<String, dynamic> json)
      : url = json['url'],
        rawBase64File = json['raw_base64_file'];
}

class TitleType {
  final String? fullString;
  final TitleTypeCode? code;
  final String? string;
  final int? episodes;
  final int? length;

  TitleType({
    this.fullString,
    this.code,
    this.string,
    this.episodes,
    this.length,
  });

  TitleType.fromJson(Map<String, dynamic> json)
      : fullString = json['full_string'],
        code =
            json['code'] == null ? null : TitleTypeCode.fromValue(json['code']),
        string = json['string'],
        episodes = json['episodes'],
        length = json['length'];
}

class Team {
  final List<String>? voice;
  final List<String>? translator;
  final List<String>? editing;
  final List<String>? decor;
  final List<String>? timing;

  Team({
    this.voice,
    this.translator,
    this.editing,
    this.decor,
    this.timing,
  });

  Team.fromJson(Map<String, dynamic> json)
      : voice = List<String>.from(json['voice']),
        translator = List<String>.from(json['translator']),
        editing = List<String>.from(json['editing']),
        decor = List<String>.from(json['decor']),
        timing = List<String>.from(json['timing']);
}

class Season {
  final String? string;
  final SeasonCode? code;
  final int? year;
  final int? weekDay;

  Season({
    this.string,
    this.code,
    this.year,
    this.weekDay,
  });

  Season.fromJson(Map<String, dynamic> json)
      : string = json['string'],
        code = json['code'] == null ? null : SeasonCode.fromValue(json['code']),
        year = json['year'],
        weekDay = json['week_day'];
}

class Franchise {
  final String? id;
  final String? name;

  Franchise({
    this.id,
    this.name,
  });

  Franchise.fromJson(Map<String, dynamic> json)
      : id = json["id"],
        name = json["name"];
}

class Blocked {
  final bool? blocked;
  final bool? wakanim;

  Blocked({
    this.blocked,
    this.wakanim,
  });

  Blocked.fromJson(Map<String, dynamic> json)
      : blocked = json['blocked'],
        wakanim = json['bakanim'];
}

class Player {
  final String? alternativePlayer;
  final String? host;
  final Episodes? episodes;
  final List<PlayerList>? list;
  final List<Rutube>? rutube;

  Player({
    this.alternativePlayer,
    this.host,
    this.episodes,
    this.list,
    this.rutube,
  });

  Player.fromJson(Map<String, dynamic> json)
      : alternativePlayer = json['alternative_player'],
        host = json['host'],
        episodes = json['episodes'] == null
            ? null
            : Episodes.fromJson(json['episodes']),
        list = json['list'] == null
            ? null
            : json['list'] is Map<String, dynamic>
                ? [for (final e in json['list'].values) PlayerList.fromJson(e)]
                : [for (final e in json['list']) PlayerList.fromJson(e)],
        rutube = json['rutube'] == null
            ? null
            : json['rutube'] is Map<String, dynamic>
                ? [for (final e in json['rutube'].values) Rutube.fromJson(e)]
                : [for (final e in json['rutube']) Rutube.fromJson(e)];
}

class Episodes {
  final int? first;
  final int? last;
  final String? string;

  Episodes({
    this.first,
    this.last,
    this.string,
  });

  Episodes.fromJson(Map<String, dynamic> json)
      : first = json['first'],
        last = json['last'],
        string = json['string'];
}

class PlayerList {
  @Deprecated(
    "use `episode` instead,"
    "this field will be removed in next major release!",
  )
  final int? serie;
  final int? episode;
  final DateTime? createdTimestamp;
  final String? preview;
  final Skips? skips;
  final Hls? hls;

  PlayerList({
    this.serie,
    this.episode,
    this.createdTimestamp,
    this.hls,
    this.skips,
    this.preview,
  });

  PlayerList.fromJson(Map<String, dynamic> json)
      : serie = json['episode'],
        episode = json['episode'],
        createdTimestamp = json['created_timestamp'] == null
            ? null
            : DateTime.fromMillisecondsSinceEpoch(
                json['created_timestamp'] * 1000),
        hls = json['hls'] == null ? null : Hls.fromJson(json['hls']),
        preview = json['preview'],
        skips = json['skips'] == null ? null : Skips.fromJson(json['skips']);
}

class Rutube {
  final DateTime? createdTimestamp;
  final int? episode;
  final String? rutubeId;

  Rutube({
    this.createdTimestamp,
    this.episode,
    this.rutubeId,
  });

  Rutube.fromJson(Map<String, dynamic> json)
      : createdTimestamp = json['created_timestamp'] == null
            ? null
            : DateTime.fromMillisecondsSinceEpoch(
                json['created_timestamp'] * 1000),
        episode = json['episode'],
        rutubeId = json['rutube_id'];
}

class Skips {
  final Skip? opening;
  final Skip? ending;

  Skips.fromJson(Map<String, dynamic> json)
      : opening = (json['opening'] ?? []).isEmpty
            ? null
            : Skip.fromTuple(json['opening']),
        ending = (json['ending'] ?? []).isEmpty
            ? null
            : Skip.fromTuple(json['ending']);
}

class Skip {
  int? start;
  int? stop;

  Skip.fromTuple(List<dynamic> tuple)
      : start = tuple[0],
        stop = tuple[1];
}

class Hls {
  final String? fhd;
  final String? hd;
  final String? sd;

  Hls({
    this.fhd,
    this.hd,
    this.sd,
  });

  Hls.fromJson(Map<String, dynamic> json)
      : fhd = json['fhd'],
        hd = json['hd'],
        sd = json['sd'];
}

class Torrents {
  final Episodes? episodes;
  final List<Torrent>? list;

  Torrents({
    this.episodes,
    this.list,
  });

  Torrents.fromJson(Map<String, dynamic> json)
      : episodes = json['episodes'] == null
            ? null
            : Episodes.fromJson(json['episodes']),
        list = json['list'] == null
            ? null
            : [for (final e in json['list']) Torrent.fromJson(e)];
}

class Torrent {
  final int? torrentId;
  final Episodes? episodes;
  final TorrentQuality? quality;
  final int? leechers;
  final int? seeders;
  final int? downloads;
  final int? totalSize;
  final String? url;
  final DateTime? uploadedTimestamp;
  final TorrentMetadata? metadata;
  final String? rawBase64File;

  Torrent({
    this.torrentId,
    this.episodes,
    this.quality,
    this.leechers,
    this.seeders,
    this.downloads,
    this.totalSize,
    this.url,
    this.uploadedTimestamp,
    this.metadata,
    this.rawBase64File,
  });

  Torrent.fromJson(Map<String, dynamic> json)
      : torrentId = json['torrent_id'],
        episodes = json['episodes'] == null
            ? null
            : Episodes.fromJson(json['episodes']),
        quality = json['quality'] == null
            ? null
            : TorrentQuality.fromJson(json['quality']),
        leechers = json['leechers'],
        seeders = json['seeders'],
        downloads = json['downloads'],
        totalSize = json['total_size'],
        url = json['url'],
        uploadedTimestamp = json['uploaded_timestamp'] == null
            ? null
            : DateTime.fromMillisecondsSinceEpoch(
                json['uploaded_timestamp'] * 1000),
        metadata = json['metadata'] == null
            ? null
            : TorrentMetadata.fromJson(json['metadata']),
        rawBase64File = json['raw_base64_file'];
}

class TorrentQuality {
  final String? string;
  final String? type;
  final String? resolution;
  final String? encoder;
  final bool? lqAudio;

  TorrentQuality({
    this.string,
    this.type,
    this.resolution,
    this.encoder,
    this.lqAudio,
  });

  TorrentQuality.fromJson(Map<String, dynamic> json)
      : string = json['string'],
        type = json['type'],
        resolution = json['resolution'],
        encoder = json['encoder'],
        lqAudio = json['lq_audio'];
}

class TorrentMetadata {
  final String? hash;
  final String? name;
  final List<String>? announce;
  final DateTime? createdTimestamp;
  final List<TorrentFile>? filesList;

  TorrentMetadata({
    this.hash,
    this.name,
    this.announce,
    this.createdTimestamp,
    this.filesList,
  });

  TorrentMetadata.fromJson(Map<String, dynamic> json)
      : hash = json['hash'],
        name = json['name'],
        announce = json['files_list'],
        createdTimestamp = json['created_timestamp'] == null
            ? null
            : DateTime.fromMillisecondsSinceEpoch(
                json['created_timestamp'] * 1000),
        filesList = json['files_list'] == null
            ? null
            : [for (final e in json['files_list']) TorrentFile.fromJson(e)];
}

class TorrentFile {
  final String? file;
  final int? size;
  final int? offset;

  TorrentFile({
    this.file,
    this.size,
    this.offset,
  });

  TorrentFile.fromJson(Map<String, dynamic> json)
      : file = json['file'],
        size = json['size'],
        offset = json['file'];
}
